unit XPButton;
{------------------------------------------------------------------------------
 Version 1.0.0

-------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, stdctrls, graphics,dialogs;

type

{ ermitteln der Version der ComCtl32.dll }
  PDLLVerInfo=^TDLLVersionInfo;
  TDLLVersionInfo=Record
	cbSize,					// Size of the structure, in bytes.
	dwMajorVersion,			// Major version of the DLL
	dwMinorVersion,			// Minor version of the DLL
	dwBuildNumber,			// Build number of the DLL
	dwPlatformID: DWord;	// Identifies the platform for which the DLL was built
  end;
{ ------------- }

  RECT = record
	left: Integer;
	top: Integer;
	right: Integer;
	bottom: Integer;
  end;

  BUTTON_IMAGELIST = record
	himl: Integer;//Pointer;
	margin: RECT;
	uAlign: Integer;
  end;

  /// <summary>
  /// The alignment of an image within a button.
  /// </summary>
  TXPButtAlignment = (Left, Right, Top, Bottom, Center);

  TXPButton = class(TButton)
  private
	FPicture: TPicture;
	FAlignment: TXPButtAlignment;
	FLeftMargin: Integer;
	FTop: Integer;
	FBottom: Integer;
	FRight: Integer;
	FCaption2: String;			// Falls wegen einer lteren Comctl32.dll das Icon
								// nicht gesetzt werden kann, wird stattdessen dieser
								// Text angezeigt
	imageList: TImageList;

	imageset: Integer;
	BCM_SETIMAGELIST: Integer;// ****
	bRect: RECT;
	bImageList: BUTTON_IMAGELIST;

	procedure SetPicture(Value: TPicture);
  protected
  public
	constructor Create(AOwner : TComponent); override;
	destructor Destroy; override;
	procedure SetImage;
  published
	property Caption2: String read FCaption2 write FCaption2;
	property Height default 25;
	property Width default 65;
	property Alignment: TXPButtAlignment read FAlignment write FAlignment;
	property Margin: Integer read FLeftMargin write FLeftMargin default -1;
	property Picture: TPicture read FPicture write SetPicture;
  end; {TXPButton}

var
 DllGetVersion: function(dvi: PDLLVerInfo): PDLLVerInfo; stdcall;
 DLLVersionInfo: TDLLVersionInfo;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Standard', [TXPButton]);
end;

function GetDllVersion(DllName: string; var DLLVersionInfo: TDLLVersionInfo): Boolean;
var
  hInstDll: THandle;
  p: pDLLVerInfo;
begin
  Result := False;
  // Get a handle to the DLL module.
  // das Handle zum DLL Modul ermitteln.
  hInstDll := LoadLibrary(PChar(DllName));
  if (hInstDll = 0) then Exit;
  // Return the address of the specified exported (DLL) function.
  // Adresse der Dll-Funktion ermitteln
  @DllGetVersion := GetProcAddress(hInstDll, 'DllGetVersion');
  // If the handle is not valid, clean up an exit.
  // Wenn das Handle ungltig ist, wird die Funktion verlassen
  if (@DllGetVersion) = nil then begin
	FreeLibrary(hInstDll);
	Exit;
  end;

  new(p);
  try
	ZeroMemory(p, SizeOf(p^));
	p^.cbSize := SizeOf(p^);

	// Call the DllGetVersion function
	// Die DllGetVersion() Funktion aufrufen
	DllGetVersion(p);
	DLLVersionInfo.dwMajorVersion := p^.dwMajorVersion;
	DLLVersionInfo.dwMinorVersion := p^.dwMinorVersion;

	@DllGetVersion := nil;
	Result := True;
  finally
	dispose(P);
  end;
 // Free the DLL module.
 // Dll wieder freigeben.
 FreeLibrary(hInstDll);
end;

{ TXPButton }

function TrueFontWidth(fnt: TFont; const text: String): Integer;
var
   dc: hdc;
   tsize : Windows.TSize;
begin
   dc := GetDC(0);
   SelectObject(DC, fnt.Handle);
   GetTextExtentPoint32(dc, PChar(text), Length(text), tsize);
   ReleaseDC(0, DC);
   Result := tsize.cx;
end;

{ Constructor }
constructor TXPButton.Create(AOwner: TComponent);
begin
	inherited Create(AOwner);
	Margin := -1;				{ default Eigenschaft }

	FPicture := TPicture.Create;

	{	Falls im Formular keine ImageList vorkommt, mu diese hier registriert werden,
		ansonsten erscheint eine "Exception der Klasse EClassNotFound" }
	RegisterClass(TImageList);

	imageList := TImageList.Create(AOwner);
	imageList.Masked := True;
	imageList.DrawingStyle := dsTransparent;
	BCM_SETIMAGELIST := $1600 + 2;
end;

{ Destructor }
destructor TXPButton.Destroy;
begin
  inherited;
	imageList.Free; imageList := nil;
	FPicture.Free; FPicture := nil;
end;

procedure TXPButton.SetPicture(Value: TPicture);
begin
	FPicture.Assign(Value);
end;

function ConvertColor(_Color: TColor): TColor;
var PixelColor, NewColor: Integer;
begin
	PixelColor := ColorToRGB(_Color);
	NewColor := Round((((PixelColor shr 16) + ((PixelColor shr 8) and $00FF) +
		(PixelColor and $0000FF)) div 3)) div 2 + 96;
	Result := RGB(NewColor, NewColor, NewColor);
end;

procedure ConvertColors(_Bitmap: TBitmap);
var x, y: Integer;
begin
	for x := 0 to _Bitmap.Width - 1 do
		for y := 0 to _Bitmap.Height - 1 do begin
			_Bitmap.Canvas.Pixels[x, y] := ConvertColor(_Bitmap.Canvas.Pixels[x, y]);
		end;
end;

procedure TXPButton.SetImage();
const MaskColor = 5757;
var x, y: Integer;
	isize: Integer;
	iconsize: Integer;

	col, grey: TColor;
	disabledBitmap: TBitmap;
	_rect: TRect;
begin
	{	Falls ComCtl32.dll Version < 6,
		dann als Ersatz fr das Image die Caption2 setzen }
	if GetDllVersion('comctl32.dll',DLLVersionInfo) then
		if (DLLVersionInfo.dwMajorVersion < 6) then begin
			Caption := FCaption2;
			Exit;
		End;

	{	image - The images for the button
		normalImage - The normal state image for the button.
		hoverImage - The hover state image for the button.
		pressedImage - The pressed state image for the button.
		disabledImage - The disabled state image for the button.
		focusedImage - The focused state image for the button. }

	imageList.Width := FPicture.Bitmap.Width;
	imageList.Height := FPicture.Bitmap.Height;

	{ Alphachannel wiederherstellen }
	x := FPicture.Bitmap.Width;
	y := FPicture.Bitmap.Height;
	col := FPicture.Bitmap.Canvas.Pixels[x-1,y-1];

	{ deaktiviertes Icon erstellen }
	disabledBitmap := TBitmap.Create;
	disabledBitmap.Canvas.Brush.Color := MaskColor;
//	_rect := Rect(0, 0, x, y);
//	disabledBitmap.Canvas.FillRect(_rect);
//	imageList.GetBitmap(0, disabledBitmap);
	disabledBitmap.Assign(FPicture.Bitmap);
	ConvertColors(disabledBitmap);
	grey := disabledBitmap.Canvas.Pixels[x-1,y-1];

	imageList.AddMasked(FPicture.Bitmap, col);	// normales Image
	imageList.AddMasked(FPicture.Bitmap, col);	// hover Image
	imageList.AddMasked(FPicture.Bitmap, col);	// pressed Image
	imageList.AddMasked(disabledBitmap, grey);	// disabled Image
	imageList.AddMasked(FPicture.Bitmap, col);	// focused Image
	disabledBitmap.FreeImage;

	{ das Icon ausrichten }
	if (FLeftMargin = -1) then begin
		isize := TrueFontWidth(Font, Caption);
		iconsize := imageList.Width;
		FLeftMargin := (ClientWidth-5-isize-iconsize) div 2;	// die "5" mu noch variabel gesetzt werden
	end;
	bRect.left := FLeftMargin; bRect.top := 0; bRect.bottom := 0; bRect.right := 0;

	bImageList.margin := bRect;
	bImageList.uAlign := Integer(FAlignment);
	bImageList.himl := imageList.Handle;
	SendMessage(Handle, BCM_SETIMAGELIST, 0, Integer(@bImageList));
end;

end.

